use bevy::math::{IVec3, UVec3};

pub trait BloxelView<T> {
    fn size(&self) -> UVec3;

    fn contains(&self, pos: impl Into<IVec3>) -> bool;

    fn get(&self, pos: impl Into<IVec3>) -> T;
}

pub trait BloxelViewMut<T>: BloxelView<T> {
    fn set(&mut self, pos: impl Into<IVec3>, value: T);
}

pub fn ivec3_to_index(pos: impl Into<IVec3>, size: UVec3) -> usize {
    let pos = pos.into();
    let size = size.as_ivec3();
    (pos.x + (pos.y * size.x) + (pos.z * size.x * size.y)) as usize
}

pub fn index_to_ivec3(index: usize, size: UVec3) -> IVec3 {
    let size = size.as_ivec3();
    let i = index as i32;
    let x = i % size.x;
    let y = i / size.x % size.y;
    let z = i / size.x / size.y;
    IVec3::new(x, y, z)
}
